const {
  getOptions
} = require('loader-utils')
const validate = require('@webpack-contrib/schema-utils')
const schema = require('./schema.json')

const importRegExp = 'import (.*) from (.*)([^]*?)'

function replaceSource(source) {
  const generateGlobalComponentExp = new RegExp(`${importRegExp}Vue\\.component\\((.*), \\1\\)`)
  const generateChildComponentExp = new RegExp(`${importRegExp}components:(\\s*){([^]*?)(:?\\s+\\1)([^]*?)}`)

  let sourceStr = ''
  const translate = (match, p1, p2, p3, p4) => {
    return `${p3}Vue.component(${p4}, () => import(${p2}))`
  }

  const translateChild = (match, p1, p2, p3, p4, p5, p6, p7) => {
    return `${p3}components:{\n${p4 + ' '}${p5.trim() ? (p6.includes(':') ? p5 + ': ' : p5 + p6 + ': ') : p1 + ': '}() => import(${p2})${p7}}`
  }

  const replaceStr = (source, reg, translate) => {
    if (!reg.test(source)) {
      sourceStr = source
      return
    }
    sourceStr = source.replace(reg, translate)
    if (reg.test(sourceStr)) {
      replaceStr(sourceStr, reg, translate)
    }
  }

  replaceStr(source, generateGlobalComponentExp, translate)
  replaceStr(sourceStr, generateChildComponentExp, translateChild)
  return sourceStr
}

function replaceWildcard(str) {
  if (!str.includes('/')) {
    str += '/**'
  }
  if (str.includes('*')) {
    str = str.replace(/\*/g, '.*')
  }
  str = str.replace(/\//g, '\\/')
  return str
}

module.exports = function (source) {
  const options = getOptions(this)

  validate({
    name: 'vue-async-loader',
    schema,
    target: options
  })

  let dirs = options.dirs || '*'

  if (dirs instanceof RegExp) {
    if (dirs.test(this.resourcePath)) {
      return replaceSource(source)
    }
  }

  if (Array.isArray(dirs)) {
    dirs = dirs.map(d => replaceWildcard(d))

    const shouldReplace = dirs.some(d => new RegExp(d).test(this.resourcePath))

    if (shouldReplace) {
      return replaceSource(source)
    }
  }
  if (typeof dirs === 'string') {
    if (new RegExp(replaceWildcard(dirs)).test(this.resourcePath)) {
      return replaceSource(source)
    }
  }
  return source
}
