const path = require('path')
const webpack = require('webpack')
const memoryfs = require('memory-fs')

module.exports = ({ file, dirs, delimiter }) => {
  const compiler = webpack({
    mode: 'development',
    context: __dirname,
    entry: file,
    output: {
      path: path.resolve(__dirname),
      filename: 'bundle.js',
    },
    module: {
      rules: [{
        test: /\.txt$/,
        use: {
          loader: path.resolve(__dirname, '../lib/loader.js'),
          options: {
            dirs,
            delimiter
          }
        }
      }]
    }
  })

  compiler.outputFileSystem = new memoryfs()

  return new Promise((resolve, reject) => {
    compiler.run((err, stats) => {
      if (err) reject(err)

      resolve([stats, compiler])
    })
  })
}
