import compiler from './compiler'
import path from 'path'
// const compiler = require('./compiler')

const prefix = './files/'

describe('Vue global component', () => {
  test('Replace vue sync components to async', async () => {
    const stats = await compiler({ file: `${prefix}component1.txt` })
    const output = stats[0].toJson().modules[0].source

    expect(output).toMatch(
      "\nVue.component('testVue1', () => import('./testVue'))"
    )
  })

  test('Replace vue muti sync components to async', async () => {
    const stats = await compiler({ file: `${prefix}component3.txt` })
    const output = stats[0].toJson().modules[0].source

    expect(output).toMatch(/Vue.component\('testVue32', \(\) => import\('\.\/testVue1'\)\)/)
    expect(output).toMatch(/Vue.component\('testVue54', \(\) => import\('\.\/testVue2'\)\)/)
  })
})

describe('Vue child component', () => {
  test('Replace vue muti sync components to async', async () => {
    const stats = await compiler({ file: `${prefix}component2.txt` })
    const output = stats[0].toJson().modules[0].source

    expect(output).toMatch(/testVue3: \(\) => import\('\.\/testVue3'\)/)
    expect(output).toMatch(/testVue5: \(\) => import\('\.\/testVue5'\)/)
    expect(output).toMatch(/testVue12: \(\) => import\('\.\/testVue4'\)/)
  })
})

describe('dir option', () => {
  test('Replace vue sync components only in dirs array', async () => {
    const web = await compiler({
      file: './index.js',
      dirs: [
        path.resolve(__dirname, 'files', 'component1.txt'),
        path.resolve(__dirname, 'files', 'component2.txt')
      ]
    })
    const bundle = web[1]
      .outputFileSystem
      .readFileSync(path.resolve(__dirname, 'bundle.js'))
      .toString('utf8')
    expect(bundle).toMatch(/testVue3: \(\) => import\('\.\/testVue3'\)/)
    expect(bundle).not.toMatch(/Vue.component\('testVue32', \(\) => import\('\.\/testVue1'\)\)/)
  })

  test('Replace vue sync components only in dirs string', async () => {
    const web = await compiler({
      file: './index.js',
      dirs: path.resolve(__dirname, 'files', 'component2.txt')
    })
    const bundle = web[1]
      .outputFileSystem
      .readFileSync(path.resolve(__dirname, 'bundle.js'))
      .toString('utf8')
    expect(bundle).toMatch(/testVue12: \(\) => import\('\.\/testVue4'\)/)
    expect(bundle).not.toMatch(/Vue.component\('testVue32', \(\) => import\('\.\/testVue1'\)\)/)
  })

  test('Replace vue sync components only in dirs regexp', async () => {
    const web = await compiler({
      file: './index.js',
      dirs: /test\/files\/component2/
    })
    const bundle = web[1]
      .outputFileSystem
      .readFileSync(path.resolve(__dirname, 'bundle.js'))
      .toString('utf8')
    expect(bundle).toMatch(/testVue12: \(\) => import\('\.\/testVue4'\)/)
    expect(bundle).not.toMatch(/Vue.component\('testVue32', \(\) => import\('\.\/testVue1'\)\)/)
  })
})


// compiler({
//   file: 'files/component2.txt'
// }).then(out => {
//   console.log(out)
// })
